﻿#######################
##
## Migrate-VirtualAdapters.ps1 1.0
##
## Author: Eric Siron
## (C) 2013 Altaro Software

<#
.SYNOPSIS
Relocates management adapters on a Hyper-V 2012+ host to another virtual switch.
.DESCRIPTION
Provides a GUI interface to migrate virtual adapters in the management operating system to an alternate virtual switch.
TCP/IP settings and access-mode VLAN settings are also transferred.
Trunk mode settings are lost.

This script must be run with administrative privileges and at least two virtual switches must be present.

.EXAMPLE
Migrate-StagingSwitchToPermanentSwitch.ps1

#>


#Requires -Version 3

#Region Environment Initialization
[void][System.Reflection.Assembly]::LoadWithPartialName("System.Windows.Forms")
[void][System.Reflection.Assembly]::LoadWithPartialName("System.Drawing")
Import-LocalizedData -BindingVariable LocalStrings -FileName Migrate-VirtualAdapters.psd1
$Hostname = $env:COMPUTERNAME
$MainFormWidth = 640
$MainFormHeight = 480
$HeaderPanelHeight = 80
$ControlPanelHeight = 60
$AvailableVirtualAdapters = @()
$ScriptsToRun = @()
#EndRegion Environment Initialization

#Region UI Object Definitions
$MainForm = New-Object System.Windows.Forms.Form
$HeaderPanel = New-Object System.Windows.Forms.Panel
$HeaderPanelDivider = New-Object System.Windows.Forms.Label
$HeaderTitle = New-Object System.Windows.Forms.Label
$HeaderDescription = New-Object System.Windows.Forms.Label
$WorkTabs = New-Object System.Windows.Forms.TabControl
$IntroTab = New-Object System.Windows.Forms.TabPage
$IntroTitle = New-Object System.Windows.Forms.Label
$IntroExplanation = New-Object System.Windows.Forms.TextBox
$SwitchTab = New-Object System.Windows.Forms.TabPage
$SwitchTitle = New-Object System.Windows.Forms.Label
$SwitchSourceLabel = New-Object System.Windows.Forms.Label
$SwitchSourceSelector = New-Object System.Windows.Forms.ComboBox
$SwitchDeleteSource = New-Object System.Windows.Forms.CheckBox
$SwitchDestinationLabel = New-Object System.Windows.Forms.Label
$SwitchDestinationSelector = New-Object System.Windows.Forms.ComboBox
$AdapterTab = New-Object System.Windows.Forms.TabPage
$AdapterTitle = New-Object System.Windows.Forms.Label
$AdapterSelectionLabel = New-Object System.Windows.Forms.Label
$AdapterSelectionBox = New-Object System.Windows.Forms.CheckedListBox
$SummaryTab = New-Object System.Windows.Forms.TabPage
$SummaryTitle = New-Object System.Windows.Forms.Label
$SummaryLabel = New-Object System.Windows.Forms.Label
$SummaryTextBox = New-Object System.Windows.Forms.TextBox
$ResultsTab = New-Object System.Windows.Forms.TabPage
$ResultsTitle = New-Object System.Windows.Forms.Label
$ResultsTextBox = New-Object System.Windows.Forms.TextBox
$ControlPanel = New-Object System.Windows.Forms.Panel
$ControlPanelDivider = New-Object System.Windows.Forms.Label
$NextButton = New-Object System.Windows.Forms.Button
$BackButton = New-Object System.Windows.Forms.Button
#EndRegion

#Region Functions
function ShowErrorDialog($Message)
{
    [System.Windows.Forms.MessageBox]::Show($Message, $ScriptTitle, [System.Windows.Forms.MessageBoxButtons]::OK, [System.Windows.Forms.MessageBoxIcon]::Error)
}
function ClickBack
{
	$BackButton.Enabled = $false
	$NextButton.Enabled = $false
	$MainForm.Cursor = [System.Windows.Forms.Cursors]::WaitCursor
	if($WorkTabs.SelectedIndex -eq 0)
	{ return }
	else
	{ $WorkTabs.SelectedIndex -= 1 }
}

function ClickNext
{
    $BackButton.Enabled = $false
    $NextButton.Enabled = $false
    $MainForm.Cursor = [System.Windows.Forms.Cursors]::WaitCursor
    if($WorkTabs.SelectedTab.Name -eq "Results")
    {
        $MainForm.Close()
    }
    else
    {
        $WorkTabs.SelectedIndex += 1
    }
}

function SetBackNextButtons
{
    if($WorkTabs.SelectedIndex -eq 0 -or $WorkTabs.SelectedIndex -eq ($WorkTabs.TabCount - 1))
    {
        $BackButton.Enabled = $false
    }
    else
    {
        $BackButton.Enabled = $true
    }
    switch ($WorkTabs.SelectedTab.Name)
    {
        "Summary" {
        	$NextButton.Text = $LocalStrings.Finish
        }
        "Results" {
        	$NextButton.Text = $LocalStrings.Close
        }
        default {
        	$NextButton.Text = $LocalStrings.Next
        }
    }
    $NextButton.Enabled = $true
    $MainForm.Cursor = [System.Windows.Forms.Cursors]::Arrow
}

function PopulateAdapters
{
	$script:AvailableVirtualAdapters = @()
	$AdapterSelectionBox.Items.Clear()
	$SwitchID = ($VSwitches[($SwitchSourceSelector.SelectedIndex)]).Id
	foreach($VAdapter in $VAdapters)
	{
		if($SwitchID -eq $VAdapter.SwitchId)
		{
			$AdapterSelectionBox.Items.Add($VAdapter.Name) | Out-Null
			$script:AvailableVirtualAdapters += ,@($VAdapter)
		}
	}
}

function IntroPageVisible
{
	if ($IntroTab.Visible)
	{
		SetBackNextButtons
		$IntroExplanation.Select(0, 0)
	}
}

function SwitchChanged
{
    PopulateAdapters
    ValidateSwitchPage
}

function SwitchPageVisible
{
    if($SwitchTab.Visible)
    {
        SetBackNextButtons
        ValidateSwitchPage
    }
}

function ValidateSwitchPage
{
    $NextButton.Enabled = !($SwitchSourceSelector.SelectedIndex -eq $SwitchDestinationSelector.SelectedIndex)
}

function AdapterPageVisible
{
    if($AdapterTab.Visible)
    { SetBackNextButtons }
}

function SummaryPageVisible
{
    if(!$SummaryTab.Visible)
    { return }
    
    $script:ScriptsToRun = @()
    $SummaryDisplayText = $LocalStrings.TaskList
    $SourceSwitch = $VSwitches[($SwitchSourceSelector.SelectedIndex)]
    $DestinationSwitch = $VSwitches[($SwitchDestinationSelector.SelectedIndex)]

    if($SwitchDeleteSource.Checked)
    {
		$SummaryDisplayText += $LocalStrings.DeleteSwitch -f $SourceSwitch.Name
		$script:ScriptsToRun += "Get-VMSwitch -Name """ + $SourceSwitch.Name + """ | Where-Object -Property Id -eq " + $SourceSwitch.Id + " | Remove-VMSwitch -Force"
    }

    foreach($AdapterIndex in $AdapterSelectionBox.CheckedIndices)
    {
        if(!$SwitchDeleteSource.Checked)
        {
			$SummaryDisplayText += $LocalStrings.RemoveVirtualAdapter -f $AvailableVirtualAdapters[$AdapterIndex].Name
            $script:ScriptsToRun += "Get-VMNetworkAdapter -ManagementOS | Where-Object -Property AdapterId -eq " + $AvailableVirtualAdapters[$AdapterIndex].AdapterId + " | Remove-VMNetworkAdapter"
        }
		$SummaryDisplayText += $LocalStrings.RecreateAdapter -f $AvailableVirtualAdapters[$AdapterIndex].Name, $DestinationSwitch.Name
		$script:ScriptsToRun += "`$NewVAdapter = Add-VMNetworkAdapter -ManagementOS -Name """ + $AvailableVirtualAdapters[$AdapterIndex].Name + """ -SwitchName """ + $DestinationSwitch.Name + """ -StaticMacAddress " + $AvailableVirtualAdapters[$AdapterIndex].MacAddress + " -Passthru"
		$script:ScriptsToRun += "`$NewHostAdapter = Get-NetAdapter | Where-Object -Property InstanceID -eq `$NewVAdapter.DeviceId"
		$script:ScriptsToRun += "`$StopOnError = `$false"
		$SourceVlanData = Get-VMNetworkAdapter -ManagementOs | Where-Object -Property AdapterId -EQ $AvailableVirtualAdapters[$AdapterIndex].AdapterId | Get-VMNetworkAdapterVlan
		if($SourceVlanData.OperationMode -eq "Access")
		{
			$SummaryDisplayText += $LocalStrings.AssignAdapter -f $AvailableVirtualAdapters[$AdapterIndex].Name, $SourceVlanData.AccessVlanId
			$script:ScriptsToRun += "Get-VMNetworkAdapter -ManagementOs | Where-Object -Property AdapterId -eq `$NewVAdapter.AdapterId | Set-VMNetworkAdapterVlan -Access -VlanId " + $SourceVlanData.AccessVlanId
		}
		elseif($SourceVlanData.OperationMode -eq "Trunk")
		{
			$SummaryDisplayText += $LocalStrings.TrunkLost -f $AvailableVirtualAdapters[$AdapterIndex].Name
		}
		$HostAdapter = Get-NetAdapter | Where-Object -Property InstanceID -eq $AvailableVirtualAdapters[$AdapterIndex].DeviceId
		$HostAdapterIPs = Get-NetIPAddress -InterfaceIndex $HostAdapter.InterfaceIndex
		$HostAdapterIPs | ForEach-Object -Process {
			if ($_.PrefixOrigin -eq "Manual")
			{
				$script:ScriptsToRun += "New-NetIPAddress -InterfaceIndex `$NewHostAdapter.InterfaceIndex -IPAddress " + $_.IPAddress + " -PrefixLength " + $_.PrefixLength
			}
		}
		$HostAdapterGateway = Get-NetRoute -InterfaceIndex $HostAdapter.InterfaceIndex | Where-Object { $_.NextHop -ne "0.0.0.0" -and $_.NextHop -ne "::" }
		$HostAdapterGateway | ForEach-Object -Process {
			$script:ScriptsToRun += "New-NetRoute -InterfaceIndex `$NewHostAdapter.InterfaceIndex -DestinationPrefix " + $_.DestinationPrefix + " -RouteMetric " + $_.RouteMetric + " -NextHop " + $_.NextHop
		}
		if((Get-NetIPInterface -InterfaceIndex $HostAdapter.InterfaceIndex).Dhcp -eq "Disabled")
		{
			$HostAdapterDNSServers = (Get-DnsClientServerAddress -InterfaceIndex $HostAdapter.InterfaceIndex).ServerAddresses
			if($HostAdapterDNSServers.Count -gt 0)
			{ $HostAdapterDNSServers = [String]::Join(",", $HostAdapterDNSServers) }
			if($HostAdapterDNSServers.Length)
			{
				$script:ScriptsToRun += "Set-DnsClientServerAddress -InterfaceIndex `$NewHostAdapter.InterfaceIndex -ServerAddresses " + $HostAdapterDNSServers
			}
		}
		$HostAdapterRegisterInDNS = [int]((Get-DnsClient -InterfaceIndex $HostAdapter.InterfaceIndex).RegisterThisConnectionsAddress)
		$script:ScriptsToRun += "Set-DnsClient -InterfaceIndex `$NewHostAdapter.InterfaceIndex -RegisterThisConnectionsAddress $HostAdapterRegisterInDNS"
	}
	$SummaryDisplayText += "`r`n`r`nPowerShell Scripts:`r`n`r`n"
	$script:ScriptsToRun | ForEach-Object { $SummaryDisplayText += $_ + "`r`n" }
	$SummaryTextBox.Text = $SummaryDisplayText
	SetBackNextButtons
	$SummaryTextBox.Select(0, 0)
}

function ResultsPageVisible
{
	if($ResultsTab.Visible)
	{
		$LineError = ""
		$ErrorText = ""
		$StopOnError = $true
		foreach($ScriptLine in $ScriptsToRun)
		{
			try
			{
				Invoke-Expression -Command $ScriptLine | Out-Null
			}
			catch
			{
				$ErrorText += $LocalStrings.ErrorLine -f $ScriptLine, $_
				if ($StopOnError)
				{ break }
			}
		}
		if($ErrorText.Length -gt 0)
		{
			$ResultsTextBox.Text = $LocalStrings.ErrorsOccurred
			$ResultsTextBox.Text += $ErrorText
		}
		else
		{ $ResultsTextBox.Text += $LocalStrings.Success }
		SetBackNextButtons
		$ResultsTextBox.Select(0, 0)
	}
}
#EndRegion Functions

#Region Main Form Construction
function BuildMainForm
{
	$MainForm.ClientSize = New-Object System.Drawing.Size($MainFormWidth, $MainFormHeight)
	$MainForm.FormBorderStyle = [System.Windows.Forms.FormBorderStyle]::Fixed3D
	$MainForm.MaximizeBox = $false
	$MainForm.Text = $LocalStrings.ScriptTitle

	$HeaderPanel.Size = New-Object System.Drawing.Size($MainFormWidth, $HeaderPanelHeight)
	$HeaderPanel.BackColor = [System.Drawing.Color]::White

	$HeaderPanelDivider.Text = ""
	$HeaderPanelDivider.AutoSize = $false
	$HeaderPanelDivider.Size = New-Object System.Drawing.Size(($HeaderPanel.ClientSize.Width), 2)
	$HeaderPanelDivider.Left = 0
	$HeaderPanelDivider.BorderStyle = [System.Windows.Forms.BorderStyle]::Fixed3D

	$HeaderTitle.Font = New-Object System.Drawing.Font("Kalinga", 12, [System.Drawing.FontStyle]::Bold)
	$HeaderTitle.AutoSize = $true
	$HeaderTitle.Text = $LocalStrings.ScriptTitle

	$HeaderDescription.Font = New-Object System.Drawing.Font("Kalinga", 10, [System.Drawing.FontStyle]::Regular)
	$HeaderDescription.AutoSize = $true
	$HeaderDescription.Text = $LocalStrings.ScriptDescription

	$WorkTabs.Width = $MainFormWidth
	$WorkTabs.SizeMode = [System.Windows.Forms.TabSizeMode]::Fixed
	$WorkTabs.ItemSize = New-Object System.Drawing.Size(0, 1)
	$WorkTabs.Appearance = [System.Windows.Forms.TabAppearance]::Buttons

	$IntroTab.Name = "Intro"
	$IntroTab.add_Enter({$NextButton.Focus()})
	$IntroTab.add_VisibleChanged({IntroPageVisible})

	$IntroTitle.Text = $LocalStrings.Introduction
	$IntroTitle.Font = New-Object System.Drawing.Font("Kalinga", 11, [System.Drawing.FontStyle]::Bold)

	$IntroExplanation.Multiline = $true
	$IntroExplanation.ReadOnly = $true
	$IntroExplanation.WordWrap = $true
	$IntroExplanation.BorderStyle = [System.Windows.Forms.BorderStyle]::None
	$IntroExplanation.Text = "{0}`r`n{1}`r`n{2}`r`n{3}" -f $LocalStrings.ScriptTitle, $LocalStrings.ScriptDescription, $LocalStrings.Copyright, $LocalStrings.IntroText

	$SwitchTab.Name = "Switches"
	$SwitchTab.add_VisibleChanged({SwitchPageVisible})

	$SwitchTitle.Text = $LocalStrings.SwitchSelection
	$SwitchTitle.Font = New-Object System.Drawing.Font("Kalinga", 11, [System.Drawing.FontStyle]::Bold)
	$SwitchTitle.AutoSize = $true

	$SwitchSourceLabel.Text = $LocalStrings.SourceSwitch
	$SwitchSourceLabel.AutoSize = $true

	$SwitchSourceSelector.DropDownStyle = [System.Windows.Forms.ComboBoxStyle]::DropDown
	$SwitchSourceSelector.add_SelectedIndexChanged({SwitchChanged})

	$SwitchDeleteSource.Text = $LocalStrings.DeleteSource
	$SwitchDeleteSource.AutoSize = $true

	$SwitchDestinationLabel.Text = $LocalStrings.DestinationSwitch
	$SwitchDestinationLabel.AutoSize = $true

	$SwitchDestinationSelector.DropDownStyle = [System.Windows.Forms.ComboBoxStyle]::DropDown
	$SwitchDestinationSelector.add_SelectedIndexChanged({ValidateSwitchPage})

	foreach ($VSwitch in $VSwitches)
	{
		$SwitchSourceSelector.Items.Add($VSwitch.Name) | Out-Null
		$SwitchDestinationSelector.Items.Add($VSwitch.Name) | Out-Null
	}
	$SwitchSourceSelector.SelectedIndex = 0
	$SwitchDestinationSelector.SelectedIndex = 1

	$AdapterTab.Name = "Adapters"
	$AdapterTab.add_VisibleChanged({AdapterPageVisible})

	$AdapterTitle.Text = $LocalStrings.AdapterSelection
	$AdapterTitle.Font = New-Object System.Drawing.Font("Kalinga", 11, [System.Drawing.FontStyle]::Bold)
	$AdapterTitle.AutoSize = $true

	$AdapterSelectionLabel.Text = $LocalStrings.AdaptersToMove
	$AdapterSelectionLabel.AutoSize = $true

	$SummaryTab.Name = "Summary"
	$SummaryTab.add_VisibleChanged({SummaryPageVisible})

	$SummaryTitle.Text = $LocalStrings.Summary
	$SummaryTitle.Font = New-Object System.Drawing.Font("Kalinga", 11, [System.Drawing.FontStyle]::Bold)
	$SummaryTitle.AutoSize = $true

	$SummaryLabel.Text = $LocalStrings.SummaryReview
	$SummaryLabel.AutoSize = $true

	$SummaryTextBox.ReadOnly = $true
	$SummaryTextBox.Multiline = $true
	$SummaryTextBox.ScrollBars = [System.Windows.Forms.ScrollBars]::Vertical
	$SummaryTextBox.BackColor = [System.Drawing.SystemColors]::Window

	$ResultsTab.Name = "Results"
	$ResultsTab.add_Enter({$NextButton.Focus()})
	$ResultsTab.add_VisibleChanged({ResultsPageVisible})

	$ResultsTitle.Text = $LocalStrings.Results
	$ResultsTitle.Font = New-Object System.Drawing.Font("Kalinga", 11, [System.Drawing.FontStyle]::Bold)
	$ResultsTitle.AutoSize = $true

	$ResultsTextBox.ReadOnly = $true
	$ResultsTextBox.Multiline = $true
	$ResultsTextBox.ScrollBars = [System.Windows.Forms.ScrollBars]::Vertical
	$ResultsTextBox.BackColor = [System.Drawing.SystemColors]::Window

	$ControlPanel.Size = New-Object System.Drawing.Size($MainForm.Width, $ControlPanelHeight)

	$ControlPanelDivider.Text = ""
	$ControlPanelDivider.AutoSize = $false
	$ControlPanelDivider.Size = New-Object System.Drawing.Size(($ControlPanel.ClientSize.Width), 2)
	$ControlPanelDivider.Left = 0
	$ControlPanelDivider.BorderStyle = [System.Windows.Forms.BorderStyle]::Fixed3D

	$BackButton.Text = $LocalStrings.Back
	$BackButton.Enabled = $false
	$BackButton.add_Click({ClickBack})

	$NextButton.Text = $LocalStrings.Next
	$NextButton.add_Click({ClickNext})

	$IntroTab.Controls.Add($IntroTitle)
	$IntroTab.Controls.Add($IntroExplanation)
	$IntroTitle.Location = New-Object System.Drawing.Point(10, 10)

	$IntroExplanation.Location = New-Object System.Drawing.Point(10, ($IntroTitle.Bottom + 10))

	$SwitchTab.Controls.Add($SwitchTitle)
	$SwitchTab.Controls.Add($SwitchSourceLabel)
	$SwitchTab.Controls.Add($SwitchSourceSelector)
	$SwitchTab.Controls.Add($SwitchDeleteSource)
	$SwitchTab.Controls.Add($SwitchDestinationLabel)
	$SwitchTab.Controls.Add($SwitchDestinationSelector)
	$SwitchTitle.Location = New-Object System.Drawing.Point(10, 10)
	$SwitchSourceLabel.Location = New-Object System.Drawing.Point(10, 40)
	$SwitchSourceSelector.Location = New-Object System.Drawing.Point(10, 60)
	$SwitchSourceSelector.Width = $MainFormWidth - 20
	$SwitchDeleteSource.Location = New-Object System.Drawing.Point(10, 85)
	$SwitchDestinationLabel.Location = New-Object System.Drawing.Point(10, 120)
	$SwitchDestinationSelector.Location = New-Object System.Drawing.Point(10, 140)
	$SwitchDestinationSelector.Width = $MainFormWidth - 20

	$AdapterTab.Controls.Add($AdapterTitle)
	$AdapterTab.Controls.Add($AdapterSelectionLabel)
	$AdapterTab.Controls.Add($AdapterSelectionBox)
	$AdapterTitle.Location = New-Object System.Drawing.Point(10, 10)
	$AdapterSelectionLabel.Location = New-Object System.Drawing.Point(10, 40)
	$AdapterSelectionBox.Location = New-Object System.Drawing.Point(10, 60)

	$SummaryTab.Controls.Add($SummaryTitle)
	$SummaryTab.Controls.Add($SummaryLabel)
	$SummaryTab.Controls.Add($SummaryTextBox)
	$SummaryTitle.Location = New-Object System.Drawing.Point (10, 10)
	$SummaryLabel.Location = New-Object System.Drawing.Point (10, 40)
	$SummaryTextBox.Location = New-Object System.Drawing.Point (10, 60)

	$ResultsTab.Controls.Add($ResultsTitle)
	$ResultsTab.Controls.Add($ResultsTextBox)
	$ResultsTitle.Location = New-Object System.Drawing.Point (10, 10)
	$ResultsTextBox.Location = New-Object System.Drawing.Point (10, 40)

	$WorkTabs.Controls.Add($IntroTab)
	$WorkTabs.Controls.Add($SwitchTab)
	$WorkTabs.Controls.Add($AdapterTab)
	$WorkTabs.Controls.Add($SummaryTab)
	$WorkTabs.Controls.Add($ResultsTab)

	$HeaderPanel.Controls.Add($HeaderTitle)
	$HeaderPanel.Controls.Add($HeaderDescription)
	$HeaderPanel.Controls.Add($HeaderPanelDivider)
	$HeaderTitle.Top = 10
	$HeaderTitle.Left = 15
	$HeaderDescription.Top = $HeaderTitle.Bottom + 2
	$HeaderDescription.Left = 15
	$HeaderPanelDivider.Location = New-Object System.Drawing.Point(0, ($HeaderPanelHeight - $HeaderPanelDivider.Height))

	$ControlPanel.Controls.Add($ControlPanelDivider)
	$ControlPanel.Controls.Add($BackButton)
	$ControlPanel.Controls.Add($NextButton)
	$ControlPanelDivider.Top = 0
	$NextButton.Top = ($ControlPanel.Height - $NextButton.Height) / 2
	$NextButton.Left = $MainFormWidth - $NextButton.Width - 15
	$BackButton.Top = $NextButton.Top
	$BackButton.Left = $NextButton.Left - $BackButton.Width - 5

	$MainForm.Controls.Add($HeaderPanel)
	$MainForm.Controls.Add($WorkTabs)
	$MainForm.Controls.Add($ControlPanel)
	$HeaderPanel.Location = New-Object System.Drawing.Point(0, 0)
	$ControlPanel.Location = New-Object System.Drawing.Point(0, ($MainFormHeight - $ControlPanel.Height))
	$WorkTabs.Location = New-Object System.Drawing.Point(0, $HeaderPanel.Bottom)
	$WorkTabs.Height = $MainFormHeight - $ControlPanel.Height - $HeaderPanelHeight
	$IntroExplanation.Size = New-Object System.Drawing.Size(($MainFormWidth - 20), ($WorkTabs.Height - $IntroExplanation.Top))
	$AdapterSelectionBox.Size = New-Object System.Drawing.Size(($MainFormWidth - 30), ($WorkTabs.Height - $AdapterSelectionBox.Top - 20))
	$SummaryTextBox.Size = New-Object System.Drawing.Size (($MainFormWidth - 30), ($WorkTabs.Height - $SummaryTextBox.Top - 20))
	$ResultsTextBox.Size = New-Object System.Drawing.Size (($MainFormWidth - 30), ($WorkTabs.Height - $ResultsTextBox.Top - 20))

	PopulateAdapters
}
#EndRegion Main Form Construction

#Region Main Script Execution Path
if(-not ([Security.Principal.WindowsPrincipal] [Security.Principal.WindowsIdentity]::GetCurrent()).IsInRole(`
	[Security.Principal.WindowsBuiltInRole] "Administrator"))
{
	ShowErrorDialog -Message $LocalStrings.NotAdmin
    return
}

try
{
	gwmi -Namespace root\virtualization\v2 -Class __NAMESPACE -ErrorAction Stop | Out-Null
}
catch
{
    ShowErrorDialog -Message $LocalStrings.NoHyperV
    return
}
if(!$ErrorCondition)
{
	$VSwitches = Get-VMSwitch
	if($VSwitches.Count -lt 2)
	{
		ShowErrorDialog -Message $LocalStrings.TooFewSwitches
		return
	}
	$VAdapters = Get-VMNetworkAdapter -ManagementOS
}
BuildMainForm
$Response = $MainForm.ShowDialog()
#EndRegion Main Script Execution Path